##########################################################################
##########################################################################
###                                                                    ###
### Computation of graphs for topological properties of Rauzy fractals ###
### As in [ST] Anne Siegel and Joerg Thuswaldner,                      ###
###            Topological properties of Rauzy fractals                ###
### (This is the default ref we use here if not specified otherwise.)  ###
###                                                                    ###
##########################################################################
##########################################################################

import itertools
def product(L):
    if not L: return iter(((),))
    return (items + (item,) for items in product(L[:-1]) for item in L[-1])

##########################################################################
##########################################################################
### Bank of examples
##########################################################################
##########################################################################

fibo = WordMorphism({1:[1,2], 2:[1]})
tribo = WordMorphism({1:[1,2], 2:[1,3], 3:[1]})
tribo2 = WordMorphism({1:[2,1], 2:[3,1], 3:[1]})
triboflip = WordMorphism({1:[1,2], 2:[3,1], 3:[1]})
squares = WordMorphism({1:[1,3,1], 2:[1], 3:[1,1,3,2]})
pasminpisot = WordMorphism({1:[3], 2:[1], 3:[2,3]})
vague = WordMorphism({1:[1,1,2,2,3], 2:[1,2,3], 3:[2]})
yack = WordMorphism({1:[3], 2:[2,3], 3:[3,3,1,2]})
yock = WordMorphism({1:[3], 2:[2,3], 3:[2,3,2,3,1]})
backgroundsub = WordMorphism({1:[3,2,1,3,1],2:[3,2,1],3:[3,2,1,3]})
connected_disconnectedsubtile = WordMorphism({1:[1,2,3,1,2],2:[1,3,2],3:[2]})
ultracheum = WordMorphism({1:[2,3],2:[1,2,3],3:[1,1,2,2,2,3,3]})
ar1 = WordMorphism({1:[1], 2:[1,2], 3:[1,3]})
ar2 = WordMorphism({1:[2,1], 2:[2], 3:[2,3]})
ar3 = WordMorphism({1:[3,1], 2:[3,2], 3:[3]})
BSTtruc = WordMorphism({1:[1,1,2], 2:[1,1,3], 3:[1]}) # [BST, Example 5.4.13]
cool_square = WordMorphism({1:[3,1], 2:[2,1], 3:[2,2,1]})
sigma0 = WordMorphism({1:[1,1,2], 2:[1,1,3], 3:[4], 4:[1]}) # The seven examples from [ST]
sigma1 = WordMorphism({1:[1,2], 2:[3], 3:[4], 4:[5], 5:[1]})
sigma1flip = WordMorphism({1:[2,1], 2:[3], 3:[4], 4:[5], 5:[1]})
sigma2 = WordMorphism({1:[2], 2:[3], 3:[1,2]})
sigma3 = WordMorphism({1:[3], 2:[2,3], 3:[3,1,2,2,3]})
sigma4 = WordMorphism({1:[1,1,1,1,2],2:[1,1,1,1,3],3:[1]})
sigma5 = WordMorphism({1:[1,2,3], 2:[1], 3:[3,1]})
sigma6 = WordMorphism({1:[1,2], 2:[3,1], 3:[1]})
blacool = WordMorphism({1:[1,2,1,3,3] ,2: [2,1,3], 3:[2,1,3,3]})

def IK(a,b): # Ito-Kimura substitutions, a >= b >= 1
    return WordMorphism({1:[1]*a+[2], 2:[1]*b+[3], 3:[1]})
def sigtarek(i):
    return WordMorphism({1:[1,2],
                         2:[1,3],
                         3:[1,1,4]*(i-1) + ([1,1,5]+[1,1,4]*i)*(i-2) + [1,1,5] + [1,1,4]*(i-1) + [1],
                         4:[1,6],
                         5:[1,1,4]*(i-3) + [1],
                         6:[1,1,4]*(i-1) + ([1,1,5]+[1,1,4]*i)*(i-3) + [1,1,5] + [1,1,4]*(i-1) + [1]})

siglol4 = WordMorphism({1:[1,3,2,4], 2:[1,3], 3:[4], 4:[1]})
siglol6 = WordMorphism({1:[2,1], 2:[3], 3:[4,3], 4:[5], 5:[6], 6:[1]})
sigma6_2 = WordMorphism({1:[1,2], 2:[3,2], 3:[4], 4:[5], 5:[6], 6:[1]})
siglol7 = WordMorphism({1:[1,3,4,2], 2:[3], 3:[4], 4:[5], 5:[6], 6:[7], 7:[1]})
siglol7flip = WordMorphism({1:[4,3,2,1], 2:[3], 3:[4], 4:[5], 5:[6], 6:[7], 7:[1]})
siglol8 = WordMorphism({1:[1,2], 2:[3], 3:[4,5], 4:[5], 5:[6], 6:[7], 7:[8], 8:[1]})
siglol9 = WordMorphism({1:[2,1], 2:[3], 3:[6,4], 4:[5,1], 5:[6], 6:[7], 7:[8], 8:[9], 9:[1]})

# substitutions with minpisot eigenvalue (look for hokkaido)
sigma4minpisot = WordMorphism({1:[1,3,2,4], 2:[1,3], 3:[4], 4:[1]}) # trouver un avec minpisot... !
sigma6minpisot = WordMorphism({1:[2], 2:[4,3], 3:[4], 4:[5,3], 5:[6], 6:[1]})
sigma7minpisot = WordMorphism({1:[2,1], 2:[3], 3:[4], 4:[2,5], 5:[6], 6:[7], 7:[1]})
sigma7minpisot_cousin = WordMorphism({1:[1,2], 2:[3], 3:[4], 4:[5], 5:[6,3], 6:[7], 7:[1]})
sigma7minpisot_cousin2 = WordMorphism({1:[1,2], 2:[3], 3:[4], 4:[5], 5:[6], 6:[7,4], 7:[1]})
sigma8minpisot = WordMorphism({1:[1,2], 2:[4,3], 3:[4], 4:[5], 5:[6], 6:[7], 7:[8], 8:[1]})
sigma10minpisot = WordMorphism({1:[1,2], 2:[2,3], 3:[4], 4:[5], 5:[6], 6:[7], 7:[8], 8:[9], 9:[10], 10:[1]})
sigma12minpisot = WordMorphism({1:[1,2],2:[2,3],3:[4],4:[5],5:[6],6:[7],7:[8],8:[6,9],9:[10],10:[11],11:[12],12:[1]})
sigma12minpisot2 = WordMorphism({1:[1,2],2:[2,3],3:[4],4:[5],5:[6,5],6:[7],7:[7,8],8:[9],9:[10],10:[11],11:[12],12:[1]})

# substitutions with overlapping "tiles"
pas_pisot_conjecture_4 = WordMorphism({1:[2,2], 2:[3,2], 3:[4], 4:[1]})
pas_pisot_conjecture_4_2 = WordMorphism({1:[4,2], 2:[4,3], 3:[4], 4:[1]})
pas_pisot_conjecture_4_2_bis = WordMorphism({1:[2,4], 2:[3], 3:[4,1], 4:[1]})
pas_pisot_conjecture_4_2_ter = WordMorphism({1:[2,3], 2:[3], 3:[4,2], 4:[1]})
pas_pisot_conjecture_4_3 = WordMorphism({1:[2,1,2], 2:[3], 3:[4], 4:[1]})
pas_pisot_conjecture_4_5 = WordMorphism({1:[2,1,4], 2:[3], 3:[4], 4:[1]})
pas_pisot_conjecture_4_6 = WordMorphism({1:[2,3,4], 2:[3], 3:[4], 4:[1]})
pas_pisot_conjecture_6 = WordMorphism({1:[2,1], 2:[3], 3:[4,3], 4:[5], 5:[6], 6:[1]})
pas_pisot_conjecture_10 = WordMorphism({1:[1,2], 2:[2,3], 3:[4], 4:[5], 5:[6], 6:[7], 7:[8], 8:[9], 9:[10], 10:[1]})
pas_pisot_conjecture_12 = WordMorphism({1:[1,2],2:[2,3],3:[4],4:[5],5:[5,6],6:[7],7:[7,8],8:[9],9:[10],10:[11],11:[12],12:[1]})
ultra_chelou_fractal_path = WordMorphism({1:[2,3], 2:[3,3], 3:[4], 4:[1]})

pas_pisot_conjecture_4_2 = WordMorphism({1:[4,2], 2:[4,3], 3:[4], 4:[1]})
minpisot5overlap = WordMorphism({1:[1,2,3], 2:[4], 3:[5], 4:[1], 5:[2,1]})
minpisot10overlap = WordMorphism({1:[1,2],2:[2,3],3:[4],4:[5],5:[6],6:[7],7:[8],8:[9],9:[10],10:[1]})
aaa = WordMorphism({1:[1,2],2:[7,3],3:[4],4:[5],5:[6],6:[7],7:[8],8:[9],9:[10],10:[1]})
barge1 = WordMorphism({1:[1,3,2,1,1,1], 2:[1,3,2,2,1,1], 3:[3,2,2,1,1]})
barge2 = WordMorphism({1:[1,6,2,1,1,1],2:[4,3,5,2,1,1],3:[3,5,2,1,1],4:[4,3,5,4,4,4],5:[1,6,2,5,4,4],6:[6,2,5,4,4]})


##########################################################################
##########################################################################
### Various values associated with sigma
##########################################################################
##########################################################################

# Substitution, alphabet, matrix
sigma = IK(5,3)
alphabet = sigma.domain().alphabet()
size_alphabet = len(alphabet) # Also the dimension the ambiant space
M = sigma.incidence_matrix()

# Dominant eigenvalue
E = M.eigenvalues() # For info only, use Er and Es only!
beta = max(E, key=abs) # Dominant eigenvalue (beta, p. 21)
deg_beta = beta.degree() # Algebraic degree of beta (d, p. 21)

# Formal elements in the number field Q(beta.minpoly())
# Dictionnary:
#   - pi(v) = v*vb
#   - eval_b(pi(v),beta_i) = <v,vbeta_i>
#   - h(pi(v)) = b*pi(v)

K.<b> = NumberField(beta.minpoly())
ub = (M.transpose()-b).kernel().basis()[0]
vb = (M-b).kernel().basis()[0]
ZOOM = vb.denominator()
if ZOOM != 1:
    print "Denominator of vbeta is not 1: ", ZOOM
#ZOOM = 7
vb = vb*ZOOM
ub = ub/(ub*vb) # Renormalize ub according to 2.2.2

def eval_b(y, x): # Function that evaluates an element of Q(b.minpoly)
    return y.lift()(x)

def eval_b_vect(v, x): # Function that evaluates a vector of Q(b.minpoly)
    return vector([eval_b(b, x) for b in v])

# Other eigenvalues and eigenvectors
ubeta = eval_b_vect(ub, beta) # Left eigenvector associated with beta (u_beta, p. 21)
vbeta = eval_b_vect(vb, beta) # Right eigenvector associated with beta (v_beta, p. 21)
Er = [beta] # Real conjugates of beta (beta, beta^(2), ..., beta^(r), p. 21)
Es = [] # Complex conjugates of (beta^(r+1), ..., beta^(r+2s), p. 21)
Lur = [ubeta] # Right eigenvectors associated with Er
Lvr = [vbeta] # Left eigenvectors associated with Er
Lus = [] # Right eigenvectors associated with Es
Lvs = [] # Left eigenvectors associated with Es
for x in [x for x in E if x.minpoly() == beta.minpoly() and x != beta]:
    if x.imag() == 0:
        Er.append(x)
        Lur.append(eval_b_vect(ub, x))
        Lvr.append(eval_b_vect(vb, x))
    elif x.imag() != 0:
        Es.append(x) # we don't force "beta^(r+s+k) = conjugate(beta^(r+k))" as in 2.2.2
        Lus.append(eval_b_vect(ub, x))
        Lvs.append(eval_b_vect(vb, x))
E = Er + Es # All conjugates of beta, in the right order
Lu = Lur + Lus # Right eigenvectors assodiated with E
Lv = Lvr + Lvs # Left eigenvectors assodiated with E

# Canonical base vectors
Le = VectorSpace(QQbar,size_alphabet).basis()

# The set of prefix-suffixes and (indexed) prefix vectors (Def. 2.5, page 24)
prefix_suffix = dict([(i,[]) for i in alphabet])
prefix_vectors_index = dict([(i,[]) for i in alphabet])
prefix_vectors = [] # Only the nonzero vectors are added!
for i in alphabet:
    w = sigma(i)
    for l in range(len(w)):
        v = vector(w[0:l].parikh_vector())
        v.set_immutable()
        prefix_suffix[i].append((w[0:l],w[l],w[l+1:]))
        prefix_vectors_index[i].append((v, w[l]))
        prefix_vectors.append(v)
prefix_vectors = list(set(prefix_vectors))

##########################################################################
##########################################################################
### Bounds
##########################################################################
##########################################################################

# Bounds in [ST, page 54] and [BST, page 276]
# With specific bound for each coordinate (Anne's trick)

prefix_bound = []
for x, v in zip(E[1:],Lv[1:]):
    a1 = max([abs(p*v) for p in prefix_vectors])
    a2 = abs(x)
    prefix_bound.append(a1/(1 - a2))

# Bounds on n_0, ..., n_{d-1} of p. 54 and 58
m = abs(max([e*(vbeta) for e in Le])) # not 1 when the beta_i are all real
Ubound  = vector([m] + prefix_bound)
Ubound2 = vector([m] + [2*x for x in prefix_bound])

N = matrix([[x^i for i in range(deg_beta)] for x in E])
B = N.inverse().norm()
coeff_bound = [floor(B*Ubound.norm())]*deg_beta
coeff_bound2 = [floor(B*Ubound2.norm())]*deg_beta

# Floating point approximations for speed-up
approx_m = m.n(1000)
approx_prefix_bound = [x.n(1000) for x in prefix_bound]
approx_prefix_bound2 = [(2*x).n(1000) for x in prefix_bound]
approx_beta = beta.n(1000)
approx_vbeta = vector(RealField(1000),vbeta)
approx_E = [x.n(1000) for x in E]

def proj_pi(x):
    x = vector(x)
    L = [x.hermitian_inner_product(v)*u for u, v in zip(Lu[1:], Lv[1:])]
    return sum(L)

##########################################################################
##########################################################################
### Pre-computation of sets of vertices
##########################################################################
##########################################################################

def compute_V():
    ### Compute V (as in (i), page 54)

    # Bounds on integer coefficients n_i of page 54 (SRS condition + bound (5.1))
    coeffs = []
    X = list(product([range(-i,i+1) for i in coeff_bound]))
    print len(X), "vectors remain after bounds on n_i"
    for x in X:
        e = sum(map(operator.mul, x, [approx_beta^i for i in range(deg_beta)]))
        if 0 <= e:
            if e < m:
                A = [abs(sum(map(operator.mul, x, [y^i for i in range(deg_beta)]))) for y in approx_E[1:]] # <x,vbeta_i>
                if all([a <= approx_prefix_bound[i] for i, a in enumerate(A)]):
                    coeffs.append(x)
    print len(coeffs), "vectors remain after SRS + bound 5.4"

    # Remove the n_i which correspond to noninteger x of Z^d

    #x1, x2, x3 = var('x1,x2,x3')
    #vv = vector((x1,x2,x3))
    #span(M2*vv) = {(n1,n2,n3) tq vb*(x1,x2,x3) = n0 + b*n1 + b^2*n2}
    #n_i tq M2.inverse()*n_i = x_i est entier

    if size_alphabet == deg_beta:
        coeffs_to_Zd = matrix(3, list(vb[0]) + list(vb[1]) + list(vb[2])).transpose().inverse()

        to_treat = list(coeffs)
        for x in to_treat:
            if (coeffs_to_Zd*vector(x)).denominator() > 1:
                coeffs.remove(x)
        print len(coeffs), "after removing n_i which correspond to noninteger x of Z^d"

    else:
        if vb[0] == 1:
            print "Didn't remove n_i which correspond to noninteger x of Z^d, but vb[0] == 1 so it should be OK"
        else:
            print "Didn't remove n_i which correspond to noninteger x of Z^d, and vb[0] > 1 so the result is probably INCORRECT"

    # Pass from coefficients n_i to elements of Z(b)
    candidates = [vector(K,x)*vector([b^n for n in range(deg_beta)]) for x in coeffs]

    # TODO add arith filter

    # Fix 0.lift() error (as of Sage 4.8)
    candidates.remove(0)
    candidates.append(0*b)

    # Condition <x,vb> < <e_i,vb> (page 54)
    V = []
    for i in alphabet:
        for x in candidates:
            a1 = eval_b(x, approx_beta)
            a2 = approx_vbeta[i-1]
            if a1 < a2:
                V.append((x,i))
    print len(V), "vertices in V"

    return V

def arith_graph_filter_boundary(L):
    ### Arithmetical filter for Ssrs and Slat

    diff_vectors = []
    for p1 in prefix_vectors:
        for p2 in prefix_vectors:
            diff_vectors.append((vb)*(vector(K,(p1)) - vector(K,(p2)))) #BBB
    diff_vectors = list(set(diff_vectors))

    # Make arithmetic graph
    G = DiGraph(loops=True)
    for g in L:
        A = [g + x for x in diff_vectors]
        A += [-x for x in A]
        for x in A:
            #print g, "\t\t", x/b # TODO
            if x/b in L:
                G.add_edge(g, x/b)

    # Remove vertices that are not the starting point of an infinite path (3 of Def 5.4)
    no_child = [v for v in G.vertices() if G.out_degree(v) == 0]
    while no_child:
        G.delete_vertices(no_child)
        no_child = [v for v in G.vertices() if G.out_degree(v) == 0]

    return G.vertices()
    #return G

def compute_Ssrs():
    ### Compute S_srs (as in 5.2.1, page 58)

    # Bounds on integer coefficients n_i of page 54, (SRS condition + bound (5.4))
    coeffs = []
    X = list(product([range(-i,i+1) for i in coeff_bound2]))
    print len(X), "vectors remain after bounds on n_i"
    for x in X:
        e = sum(map(operator.mul, x, [approx_beta^i for i in range(deg_beta)])) # <x,vbeta>
        if 0 <= e:
            if e < m:
                A = [abs(sum(map(operator.mul, x, [y^i for i in range(deg_beta)]))) for y in approx_E[1:]] # <x,vbeta_i>
                if all([a <= approx_prefix_bound2[i] for i, a in enumerate(A)]):
                    coeffs.append(x)
    print len(coeffs), "vectors remain after SRS + bound 5.4"

    # Remove the n_i which correspond to noninteger x of Z^d

    #x1, x2, x3 = var('x1,x2,x3')
    #vv = vector((x1,x2,x3))
    #span(M2*vv) = {(n1,n2,n3) tq vb*(x1,x2,x3) = n0 + b*n1 + b^2*n2}
    #n_i tq M2.inverse()*n_i = x_i est entier

    if size_alphabet == deg_beta:
        coeffs_to_Zd = matrix(3, list(vb[0]) + list(vb[1]) + list(vb[2])).transpose().inverse()

        to_treat = list(coeffs)
        for x in to_treat:
            if (coeffs_to_Zd*vector(x)).denominator() > 1:
                coeffs.remove(x)
        print len(coeffs), "after removing n_i which correspond to noninteger x of Z^d"

    else:
        if vb[0] == 1:
            print "Didn't remove n_i which correspond to noninteger x of Z^d, but vb[0] == 1 so it should be OK"
        else:
            print "Didn't remove n_i which correspond to noninteger x of Z^d, and vb[0] > 1 so the result is probably INCORRECT"

    # Pass from coefficients n_i to elements of Z(b)
    candidates = [vector(K,x)*vector([b^n for n in range(deg_beta)]) for x in coeffs]

    # Arithmetic filter
    #return arith_graph_filter_boundary(candidates)
    candidates = arith_graph_filter_boundary(candidates)
    print len(candidates), "vectors remain after arithmetic filter"

    # Fix 0.lift() error (as of Sage 4.8)
    candidates.remove(0)
    candidates.append(0*b)

    # Conditions <x,vb> < <e_j,vb> and "x or (i < j)" (page 56)
    V = []
    for i in alphabet:
        for j in alphabet:
            for x in candidates:
                a1 = eval_b(x, approx_beta) # Approx
                a2 = Le[j-1]*approx_vbeta # Approx
                if a2 - a1 > 0.0000000000000001: # Approx
                    if x or (i < j):
                        V.append((i,x,j))
    print len(V), "vertices in S_srs"

    return V

##########################################################################
##########################################################################
### Graphs
##########################################################################
##########################################################################

def zero_expansion_graph(V):
    ### Zero-expansion graph (page 55)
    G = DiGraph(loops=True)
    G.add_vertices(V)

    # Add edges
    for g2, i2 in V:
        L = [(b*g2 - vb*vector(K, y[0]), y[1]) for y in prefix_vectors_index[i2]]
        for g1, i1 in L:
            if (g1, i1) in V:
                G.add_edge((g1,i1),(g2,i2))

    # Remove vertices that are not the starting point of an infinite path
    no_child = [v for v in G.vertices() if G.out_degree(v) == 0]
    while no_child:
        G.delete_vertices(no_child)
        no_child = [v for v in G.vertices() if G.out_degree(v) == 0]

    # Plot nicely
    G.set_pos(G.layout_default())

    return G

def boundary_graph(S):
    ### Boundary graph (page 56)
    G = DiGraph(loops=True, multiedges=True)
    G.add_vertices(S)

    #BBB
    diff_vectors = []
    for p1 in prefix_vectors:
        for p2 in prefix_vectors:
            diff_vectors.append(vb*(vector(K,(p1)) - vector(K,(p2))))
    diff_vectors = list(set(diff_vectors))

    diff_type1 = {}
    for i1 in alphabet:
        for j1 in alphabet:
            diff_type1[i1,j1] = []
            for i2 in alphabet:
                for j2 in alphabet:
                    for pi, si in [(pi, si) for pi, i, si in prefix_suffix[i2] if i == i1]:
                        for pj, sj in [(pj, sj) for pj, j, sj in prefix_suffix[j2] if j == j1]:
                            g = vb*(vector(K,(pj.parikh_vector())) - vector(K,(pi.parikh_vector())))
                            diff_type1[i1,j1].append((i2, j2, g, vb*vector(K,(pi.parikh_vector()))))
    diff_type2 = {}
    for i1 in alphabet:
        for j1 in alphabet:
            diff_type2[i1,j1] = []
            for i2 in alphabet:
                for j2 in alphabet:
                    for pi, si in [(pi, si) for pi, i, si in prefix_suffix[j2] if i == i1]:
                        for pj, sj in [(pj, sj) for pj, j, sj in prefix_suffix[i2] if j == j1]:
                            g = vb*(vector(K,(pj.parikh_vector())) - vector(K,(pi.parikh_vector())))
                            diff_type2[i1,j1].append((i2, j2, g, vb*vector(K,(pj.parikh_vector()))))

    possible_vectors = [x[1] for x in S]

    # Add edges (as 2. of Def 5.4) (edge from (i1,g1,j1) to (i2,g2,j2))
    for i1, g1, j1 in S:

        vertex_added = False

        for i2, j2, x, label in diff_type1[i1,j1]:
            a = (g1+x)/b
            if a in possible_vectors:
                G.add_edge((i1,g1,j1), (i2,a,j2), label=(label,1))
                vertex_added = True

        for i2, j2, x, label in diff_type2[i1,j1]:
            a = -(g1+x)/b
            if a in possible_vectors:
                G.add_edge((i1,g1,j1), (i2,a,j2), label=(label+g1,2))
                vertex_added = True

        """ OLD VERSION
        for i2, g2, j2 in S:
            # Type 1
            for pi, si in [(pi, si) for pi, i, si in prefix_suffix[i2] if i == i1]:
                for pj, sj in [(pj, sj) for pj, j, sj in prefix_suffix[j2] if j == j1]:
                    a1 = b*g2
                    a2 = g1 + vb*(vector(K,(pj.parikh_vector())) - vector(K,(pi.parikh_vector())))
                    if a1 == a2:
                        G.add_edge((i1,g1,j1), (i2,g2,j2), label=(vb*vector(K,(pi.parikh_vector())),1))
                        vertex_added = True
            # Type 2
            for pi, si in [(pi, si) for pi, i, si in prefix_suffix[j2] if i == i1]:
                for pj, sj in [(pj, sj) for pj, j, sj in prefix_suffix[i2] if j == j1]:
                    a1 = b*g2
                    a2 = g1 + vb*(vector(K,(pj.parikh_vector())) - vector(K,(pi.parikh_vector())))
                    if a1 == -a2:
                        G.add_edge((i1,g1,j1), (i2,g2,j2), label=(vb*vector(K,(pj.parikh_vector())) + g1,2))
                        vertex_added = True
        """

        # Delete vertices with no outgoing edges
        if not vertex_added:
            G.delete_vertex((i1,g1,j1))
            print "deleted:", i1, g1, j1

    # Remove vertices that are not the starting point of an infinite path (3 of Def 5.4)
    no_child = [v for v in G.vertices() if G.out_degree(v) == 0]
    while no_child:
        print "deleted", len(no_child), "vertices"
        G.delete_vertices(no_child)
        no_child = [v for v in G.vertices() if G.out_degree(v) == 0]

    # Plot nicely
    G.set_pos(G.layout_default())

    return G

def tile_connectedness_graph(G):
    ### Tile-connectedness graph (page 64) TODO
    H = Graph()
    for i, g, j in G.vertices():
        if not g:
            H.add_edge((i,j))
    return H

def tile_refinement_connectedness_graph(G, i):
    ### Tile-refinement-connectedness graph (page 64)
    H = Graph()
    for j in alphabet:
        for p, i0, s in prefix_suffix[j]:
            if i0 == i:
                H.add_vertex((j, vb*vector(K,p.parikh_vector())))

    for j1, g1 in H.vertices():
        for j2, g2 in H.vertices():
            a = (g2-g1)/b
            if ((j1,a,j2) in G) or ((j2,-a,j1) in G):
                H.add_edge((j1,g1),(j2,g2))
    return H
